# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_atyp.ipynb.

# %% auto 0
__all__ = ['T', 'P', 'Mixin', 'MetaClass']

# %% ../nbs/01_atyp.ipynb 6
from abc import ABCMeta
from enum import EnumMeta

# %% ../nbs/01_atyp.ipynb 8
from typing import (Any, Type, Union, Tuple, TypeAlias, Callable, ParamSpec, TypeVar)

# %% ../nbs/01_atyp.ipynb 10
#| export


# %% ../nbs/01_atyp.ipynb 12
#| export


# %% ../nbs/01_atyp.ipynb 14
#| export


# %% ../nbs/01_atyp.ipynb 16
T = TypeVar('T')
P = ParamSpec('P')

Mixin: TypeAlias = Tuple[Type, Callable[P, Any]]

MetaClass: TypeAlias = Type[Union[T, EnumMeta, ABCMeta]]

# %% ../nbs/01_atyp.ipynb 18
#| export
