# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_cons.ipynb.

# %% auto 0
__all__ = ['TORCH_NN']

# %% ../nbs/00_cons.ipynb 6
#| export


# %% ../nbs/00_cons.ipynb 8
#| export


# %% ../nbs/00_cons.ipynb 10
#| export


# %% ../nbs/00_cons.ipynb 12
from nlit import SUFFIX, MODULE, DEFAULT, _MEMBER_NAMES_, _MEMBER_MAP_
from uscr import private

# %% ../nbs/00_cons.ipynb 14
#| export


# %% ../nbs/00_cons.ipynb 16
TORCH_NN = 'torch.nn'

# %% ../nbs/00_cons.ipynb 17
_MODULE = private(MODULE)
_DEFAULT = private(DEFAULT)
_SUFFIX = private(SUFFIX)

# %% ../nbs/00_cons.ipynb 19
#| export
